#!/usr/bin/env perl

use strict;
use warnings;
use FindBin;
use Getopt::Long qw(:config no_ignore_case bundling); #ѧϰ
use List::Util qw (shuffle);
# ˳Դtrinityԭbutterfly_commands.adj(ÿһ)	
# һһlinux̣ע̺һ˼
# Ҫһcmd_process_forker.plͬ
my $usage = <<_EOUSAGE_;

#################################################################################
#  --cmds_file	a file containing a list of commands to execute, one cmd per line.  
#  --suffix	the suffiex name of the input data file, fastq or clean       
#  --CPU	Default: 4
#  --shuffle	randomly orders the commands before executing them.
###################################################################################

_EOUSAGE_
	;

#################
##   ȫֱ  ##
#################
my $cmds_file = "";
my $suffix = "fastq";
my $CPU = 4;
my $shuffle_flag = 0;

my $ENCOUNTERED_FAILURE = 0; #ȫֱӳм̳
#################
## ##
#################
&GetOptions (
			 "cmds_file=s" => \$cmds_file,#Ҫִеڵļ
			 "suffix=s" => \$suffix,    #ļĺ׺
			 "CPU=i" => \$CPU,    #CPU
             "shuffle" => \$shuffle_flag,#ĬҪshuffle
             );

unless ($cmds_file) {
	die $usage;
}

# ־Ŀ¼־ļcmdн̼ͨѶļȱǣûûдȨ޵Ļ
my $log_dir = "cmds_log.$$";
mkdir($log_dir) or die "Error, cannot mkdir $log_dir";

#################
##  ʼ ##
#################
main: {
    mkdir("fastqc_result");
	my %job_tracker;		# ¼Ҫٵ
	my @failed_jobs;		# ¼ʧܵ
	my $num_running = 0;	
    my @unix_cmds;  #¼е
	
    #ļж	
	open (my $fh, $cmds_file) or die "Error, cannot open file $cmds_file";      
	while (<$fh>) {
		chomp;
		my @cols = split(/\t/, $_);
		my $cmd="./FastQC/fastqc -o ./fastqc_result $cols[0].$suffix"; #Ҫcmd_process_forker.plͬ
		push (@unix_cmds, $cmd);#һ洢
	}
	close $fh;
	
	#˳Ŀǰʱ䳤ģǰͶ̵ں󣩾һ
    if ($shuffle_flag) {
        @unix_cmds = shuffle(@unix_cmds);
    }
	
    my $cmd_counter = 0;
    foreach my $cmd (@unix_cmds) {	# ִ
        $cmd_counter++;      
		$num_running++;	#¼ǰе
        
		my $pid = fork();	#ÿһΪӽ̺͸̳pidȫһ븸ڵĿִжȴ
		if ($pid) { #$pidֵ0ǰṆ̀ȵ̬forkӽ
			$job_tracker{$cmd_counter} = $cmd; #¼пʼеĽ
		}
		else {#$pidֵΪ0ǰӽ̣ȵ̬cmdȻ˳ûû
			my $ret = &run_cmd($cmd_counter, $cmd);
			exit($ret);  #ִһʧܣ˳ǰ
		}
		
		if ($num_running >= $CPU) { #ѷcpu,">"ԲҪ
			wait(); # ͣǰִ
			my $num_finished = &collect_jobs(\%job_tracker, \@failed_jobs);   #ͳͳѾɹʧܵḶ̌%job_tracker
			$num_running -= $num_finished; # еĽȥѾɵģ
           		
            #⽩ʬ(Zombie process)Ϊʲôôд 
            for (1..$num_finished-1) {
                wait();
            }
           
		}
	} # ִ
	
	# жwait()ķֱֵûӽ
	while (wait() != -1) { };	#Ҫwait() ֵΪӽIDûӽ̣-1
								#ִн󼸸̣ǡõڻСڹ涨̣ӽڴ˴ȴ
	&collect_jobs(\%job_tracker, \@failed_jobs); #Ͻ̽л	
	# ȫϣɾlogļ
	`rm -rf $log_dir`;
	
	
	# ǽ	
	my $num_failed_jobs = scalar @failed_jobs; #¼ʧ
	if (! $ENCOUNTERED_FAILURE) {#־Ϊ0ʾȫɹ
		print "\nAll $cmd_counter jobs completed successfully! \n";
		exit(0);
	}
	else {# ־Ϊ0
        unless ($num_failed_jobs == $ENCOUNTERED_FAILURE) {
            print "\n\nError, $ENCOUNTERED_FAILURE jobs failed, but only have recorded $num_failed_jobs ... very strange and unexplained.\n\n";
            # I haven't seen this in my testing, but it's clear that others have, and I haven't figured out why or how yet...  bhaas
        }
        
		#ʧܵȫдһļԺֻҪЩȫ
		my $failed_cmd_file = "failed_cmds.$$.txt";
		open (my $ofh, ">$failed_cmd_file") or die "Error, cannot write to $failed_cmd_file"; #ջһΪfailed_cmds.$$.txtļΪʧ˵
		@failed_jobs = sort {$a->{index}<=>$b->{index}} @failed_jobs;  #䰴index
        foreach my $failed_job (@failed_jobs) {
            print $ofh $failed_job->{cmd} . "\n";
        }      
		close $ofh;		
		print "\n\nSorry, $num_failed_jobs of $cmd_counter jobs failed.\n\n"
			. "Failed commands written to file: $failed_cmd_file\n\n";
		exit(1);
	}
}


#################
##  ӳ     ##
#################

sub run_cmd {
	my ($index, $cmd) = @_;
	print "\nRUNNING: $cmd\n";	
	my $ret = system($cmd);		
	if ($ret) {  #㼴Ϊ
		print STDERR "Error, command: $cmd died with ret $ret";
	}		
	open (my $log_fh, ">$log_dir/$index.ret") or die "Error, cannot write to log file for $index.ret"; #һļ¼һ̵н
	print $log_fh $ret; #ⲿ'0''1'־ļ
	close $log_fh;
	return($ret); #ظ
}


sub collect_jobs {
	my ($job_tracker_href, $failed_jobs_aref) = @_;
	my @job_indices = keys %$job_tracker_href;
	my $num_finished = 0;
	foreach my $index (@job_indices) {# ÿμһ̣job	
		my $log_file = "$log_dir/$index.ret";	# һ̵нӦһļ
		if (-s $log_file) { # logҴСΪ
			my $ret_val = `cat $log_file`; # ļ
			chomp $ret_val; # ȥз
			my $job = $job_tracker_href->{$index};
			if ($ret_val == 0) {	#Ϊַ'0'̳ɹ
				print "SUCCESS[$index]: $job\n";				
			}
			else {#򣬽ʧܣҲ
				print "FAILED[$index]: $job\n";
				$ENCOUNTERED_FAILURE++;   # ¼ʧܽĽ
                push (@$failed_jobs_aref, {index => $index,
                                           cmd => $job_tracker_href->{$index},
                      });
			}			
			unlink $log_file;
			$num_finished++; # ¼Ѿɵ
            delete $job_tracker_href->{$index}; # һɣͲtrack
		}
	}

	return($num_finished);
}

